/* $Id$ */

/* SSLCipherCheck
 * Copyright (C) 2003 Jostein Tveit <josteitv@pvv.ntnu.no>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

enum protocols {
  SSL2 = 0,
  SSL3,
  TLS1
};

struct cipher {
  int protocol;
  char *cipher;
  char *bits;
};

struct cipher ssl2ciphers[] = {
  { SSL2, "SSL_CK_RC4_128_WITH_MD5",                   "\x01\x00\x80" },
  { SSL2, "SSL_CK_RC4_128_EXPORT40_WITH_MD5",          "\x02\x00\x80" },
  { SSL2, "SSL_CK_RC2_CBC_128_CBC_WITH_MD5",           "\x03\x00\x80" },
  { SSL2, "SSL_CK_RC2_CBC_128_CBC_EXPORT40_WITH_MD5",  "\x04\x00\x80" },
  { SSL2, "SSL_CK_IDEA_128_CBC_WITH_MD5",              "\x05\x00\x80" },
  { SSL2, "SSL_CK_DES_64_CBC_WITH_MD5",                "\x06\x00\x40" },
  { SSL2, "SSL_CK_DES_192_EDE3_CBC_WITH_MD5",          "\x07\x00\xC0" },
};

struct cipher ssl3ciphers[] = {
  /* SSL 3.0 <draft-freier-ssl-version3-02.txt> */
  { SSL3, "SSL_NULL_WITH_NULL_NULL",                  "\x00\x00" },
  { SSL3, "SSL_RSA_WITH_NULL_MD5",                    "\x00\x01" },
  { SSL3, "SSL_RSA_WITH_NULL_SHA",                    "\x00\x02" },
  { SSL3, "SSL_RSA_EXPORT_WITH_RC4_40_MD5",           "\x00\x03" },
  { SSL3, "SSL_RSA_WITH_RC4_128_MD5",                 "\x00\x04" },
  { SSL3, "SSL_RSA_WITH_RC4_128_SHA",                 "\x00\x05" },
  { SSL3, "SSL_RSA_EXPORT_WITH_RC2_CBC_40_MD5",       "\x00\x06" },
  { SSL3, "SSL_RSA_WITH_IDEA_CBC_SHA",                "\x00\x07" },
  { SSL3, "SSL_RSA_EXPORT_WITH_DES40_CBC_SHA",        "\x00\x08" },
  { SSL3, "SSL_RSA_WITH_DES_CBC_SHA",                 "\x00\x09" },
  { SSL3, "SSL_RSA_WITH_3DES_EDE_CBC_SHA",            "\x00\x0A" },
  { SSL3, "SSL_DH_DSS_EXPORT_WITH_DES40_CBC_SHA",     "\x00\x0B" },
  { SSL3, "SSL_DH_DSS_WITH_DES_CBC_SHA",              "\x00\x0C" },
  { SSL3, "SSL_DH_DSS_WITH_3DES_EDE_CBC_SHA",         "\x00\x0D" },
  { SSL3, "SSL_DH_RSA_EXPORT_WITH_DES40_CBC_SHA",     "\x00\x0E" },
  { SSL3, "SSL_DH_RSA_WITH_DES_CBC_SHA",              "\x00\x0F" },
  { SSL3, "SSL_DH_RSA_WITH_3DES_EDE_CBC_SHA",         "\x00\x10" },
  { SSL3, "SSL_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x11" },
  { SSL3, "SSL_DHE_DSS_WITH_DES_CBC_SHA",             "\x00\x12" },
  { SSL3, "SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA",        "\x00\x13" },
  { SSL3, "SSL_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x14" },
  { SSL3, "SSL_DHE_RSA_WITH_DES_CBC_SHA",             "\x00\x15" },
  { SSL3, "SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA",        "\x00\x16" },
  { SSL3, "SSL_DH_anon_EXPORT_WITH_RC4_40_MD5",       "\x00\x17" },
  { SSL3, "SSL_DH_anon_WITH_RC4_128_MD5",             "\x00\x18" },
  { SSL3, "SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x19" },
  { SSL3, "SSL_DH_anon_WITH_DES_CBC_SHA",             "\x00\x1A" },
  { SSL3, "SSL_DH_anon_WITH_3DES_EDE_CBC_SHA",        "\x00\x1B" },
  { SSL3, "SSL_FORTEZZA_KEA_WITH_NULL_SHA",           "\x00\x1C" },
  { SSL3, "SSL_FORTEZZA_KEA_WITH_FORTEZZA_CBC_SHA",   "\x00\x1D" },
  { SSL3, "SSL_FORTEZZA_KEA_WITH_RC4_128_SHA",        "\x00\x1E" },
};
    
struct cipher tls1ciphers[] = {
  /* TLS 1.0 (RFC 2246) */
  { TLS1, "TLS_NULL_WITH_NULL_NULL",                  "\x00\x00" },
  { TLS1, "TLS_RSA_WITH_NULL_MD5",                    "\x00\x01" },
  { TLS1, "TLS_RSA_WITH_NULL_SHA",                    "\x00\x02" },
  { TLS1, "TLS_RSA_EXPORT_WITH_RC4_40_MD5",           "\x00\x03" },
  { TLS1, "TLS_RSA_WITH_RC4_128_MD5",                 "\x00\x04" },
  { TLS1, "TLS_RSA_WITH_RC4_128_SHA",                 "\x00\x05" },
  { TLS1, "TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5",       "\x00\x06" },
  { TLS1, "TLS_RSA_WITH_IDEA_CBC_SHA",                "\x00\x07" },
  { TLS1, "TLS_RSA_EXPORT_WITH_DES40_CBC_SHA",        "\x00\x08" },
  { TLS1, "TLS_RSA_WITH_DES_CBC_SHA",                 "\x00\x09" },
  { TLS1, "TLS_RSA_WITH_3DES_EDE_CBC_SHA",            "\x00\x0A" },
  { TLS1, "TLS_DH_DSS_EXPORT_WITH_DES40_CBC_SHA",     "\x00\x0B" },
  { TLS1, "TLS_DH_DSS_WITH_DES_CBC_SHA",              "\x00\x0C" },
  { TLS1, "TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA",         "\x00\x0D" },
  { TLS1, "TLS_DH_RSA_EXPORT_WITH_DES40_CBC_SHA",     "\x00\x0E" },
  { TLS1, "TLS_DH_RSA_WITH_DES_CBC_SHA",              "\x00\x0F" },
  { TLS1, "TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA",         "\x00\x10" },
  { TLS1, "TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x11" },
  { TLS1, "TLS_DHE_DSS_WITH_DES_CBC_SHA",             "\x00\x12" },
  { TLS1, "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA",        "\x00\x13" },
  { TLS1, "TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x14" },
  { TLS1, "TLS_DHE_RSA_WITH_DES_CBC_SHA",             "\x00\x15" },
  { TLS1, "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA",        "\x00\x16" },
  { TLS1, "TLS_DH_anon_EXPORT_WITH_RC4_40_MD5",       "\x00\x17" },
  { TLS1, "TLS_DH_anon_WITH_RC4_128_MD5",             "\x00\x18" },
  { TLS1, "TLS_DH_anon_EXPORT_WITH_DES40_CBC_SHA",    "\x00\x19" },
  { TLS1, "TLS_DH_anon_WITH_DES_CBC_SHA",             "\x00\x1A" },
  { TLS1, "TLS_DH_anon_WITH_3DES_EDE_CBC_SHA",        "\x00\x1B" },

  /*  Addition of Kerberos Cipher Suites to TLS (RFC 2712) */
  { TLS1, "TLS_KRB5_WITH_DES_CBC_SHA",                "\x00\x1E" },
  { TLS1, "TLS_KRB5_WITH_3DES_EDE_CBC_SHA",           "\x00\x1F" },
  { TLS1, "TLS_KRB5_WITH_RC4_128_SHA",                "\x00\x20" },
  { TLS1, "TLS_KRB5_WITH_IDEA_CBC_SHA",               "\x00\x21" },
  { TLS1, "TLS_KRB5_WITH_DES_CBC_MD5",                "\x00\x22" },
  { TLS1, "TLS_KRB5_WITH_3DES_EDE_CBC_MD5",           "\x00\x23" },
  { TLS1, "TLS_KRB5_WITH_RC4_128_MD5",                "\x00\x24" },
  { TLS1, "TLS_KRB5_WITH_IDEA_CBC_MD5",               "\x00\x25" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_DES_CBC_40_SHA",      "\x00\x26" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_RC2_CBC_40_SHA",      "\x00\x27" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_RC4_40_SHA",          "\x00\x28" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_DES_CBC_40_MD5",      "\x00\x29" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_RC2_CBC_40_MD5",      "\x00\x2A" },
  { TLS1, "TLS_KRB5_EXPORT_WITH_RC4_40_MD5",          "\x00\x2B" },

  /* AES Ciphersuites for TLS (RFC 3268) */
  { TLS1, "TLS_RSA_WITH_AES_128_CBC_SHA",             "\x00\x2F" },
  { TLS1, "TLS_DH_DSS_WITH_AES_128_CBC_SHA",          "\x00\x30" },
  { TLS1, "TLS_DH_RSA_WITH_AES_128_CBC_SHA",          "\x00\x31" },
  { TLS1, "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",         "\x00\x32" },
  { TLS1, "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",         "\x00\x33" },
  { TLS1, "TLS_DH_anon_WITH_AES_128_CBC_SHA",         "\x00\x34" },
  { TLS1, "TLS_RSA_WITH_AES_256_CBC_SHA",             "\x00\x35" },
  { TLS1, "TLS_DH_DSS_WITH_AES_256_CBC_SHA",          "\x00\x36" },
  { TLS1, "TLS_DH_RSA_WITH_AES_256_CBC_SHA",          "\x00\x37" },
  { TLS1, "TLS_DHE_DSS_WITH_AES_256_CBC_SHA",         "\x00\x38" },
  { TLS1, "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",         "\x00\x39" },
  { TLS1, "TLS_DH_anon_WITH_AES_256_CBC_SHA",         "\x00\x3A" },

  /* ECC Cipher Suites for TLS <draft-ietf-tls-ecc-04.txt> */
  { TLS1, "TLS_ECDH_ECDSA_WITH_NULL_SHA",             "\x00\x47" },
  { TLS1, "TLS_ECDH_ECDSA_WITH_RC4_128_SHA",          "\x00\x48" },
  { TLS1, "TLS_ECDH_ECDSA_WITH_DES_CBC_SHA",          "\x00\x49" },
  { TLS1, "TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA",     "\x00\x4A" },
  { TLS1, "TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA",      "\x00\x4B" },
  { TLS1, "TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA",      "\x00\x4C" },

  /* 56-bit Export Cipher Suites for TLS <draft-ietf-tls-56-bit-ciphersuites-01.txt> */
  { TLS1, "TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA",      "\x00\x62" },
  { TLS1, "TLS_RSA_EXPORT1024_WITH_RC4_56_SHA",       "\x00\x64" },
  { TLS1, "TLS_DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA",  "\x00\x63" },
  { TLS1, "TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA",   "\x00\x65" },
  { TLS1, "TLS_DHE_DSS_WITH_RC4_128_SHA",             "\x00\x66" },

};
